﻿EnableExplicit

;- Information.
; Dieses Module ist das Hauptfenster des Projekts.

;- Version.
; Status:   In Arbeit.
; Autor:    Peter Dreisiebner
; Website:  http://peter.dreisiebner.at
; E-Mail:   web@dreisiebner.at
; Geändert: 2017-03-29 - PureBasic v5.60 (x86).


;-
;- DeclareModule WindowStart.
;-
DeclareModule WindowStart
   EnableExplicit

   Declare.i Open()

EndDeclareModule  ; WindowStart.

;
;- Include.
;

;-
;- Module WindowStart.
;-
Module WindowStart
   EnableExplicit
   UseModule Common
   DebugOutWindowStart

   ;- Konstanten.
   #iWindowWidth = 800
   #iWindowHeight = 450
   CompilerIf #fDebugOut Or #fDebugSpeed
      #sWindowTitle = #sApplicationName + " (Debugversion)"
   CompilerElse
      #sWindowTitle = #sApplicationName
   CompilerEndIf

   #sMenuTitleFile = "&Datei"
   #sMenuItemFileExit = "&Beenden"
   #sMenuTitleEdit = "&Bearbeiten"
   #sMenuTitleHelp = "&Hilfe"
   #sMenuItemHelpLog = "&Log..."
   #sMenuItemHelpAbout = "&Über..."
   Enumeration eMenuItemId
      #eMenuItemFileExit
      #eMenuItemHelpLog
      #eMenuItemHelpAbout
   EndEnumeration

   Global miWindowNumber.i
   Global miMenuNumber.i

   Declare Close()


   Procedure.i EventCloseWindow()
      ; Ereignisprozedur wenn das Fenster geschlossen werden soll.
      DebugView::ProcIn()

      Close()

      DebugView::ProcOut()
   EndProcedure


   Procedure EventMinimizeWindow()
      DebugView::ProcIn()

      WindowLog::Hide()

      DebugView::ProcOut()
   EndProcedure


   Procedure EventRestoreWindow()
      DebugView::ProcIn()

      WindowLog::RestoreWindow()

      DebugView::ProcOut()
   EndProcedure


   Procedure EventMenuFileExit()
      DebugView::ProcIn()

      Close()

      DebugView::ProcOut()
   EndProcedure


   Procedure EventMenuHelpLog()
      DebugView::ProcIn()

      WindowLog::Show(miWindowNumber)

      DebugView::ProcOut()
   EndProcedure


   Procedure EventMenuHelpAbout()
      DebugView::ProcIn()

      WindowAbout::Open(miWindowNumber)

      DebugView::ProcOut()
   EndProcedure


   Procedure.i Open()
      ; Das Programmfenster laden und anzeigen.
      DebugView::ProcIn()

      miWindowNumber = OpenWindow(#PB_Any, #PB_Ignore, #PB_Ignore, #iWindowWidth, #iWindowHeight, #sWindowTitle,
                                  #PB_Window_MinimizeGadget | #PB_Window_MaximizeGadget | #PB_Window_SizeGadget)
      If miWindowNumber
         BindEvent(#PB_Event_CloseWindow, @EventCloseWindow(), miWindowNumber)
         BindEvent(#PB_Event_MinimizeWindow, @EventMinimizeWindow(), miWindowNumber)
         BindEvent(#PB_Event_RestoreWindow, @EventRestoreWindow(), miWindowNumber)

         Tool::WindowReduceFlicker(miWindowNumber)

         miMenuNumber = CreateMenu(#PB_Any, WindowID(miWindowNumber))
         If miMenuNumber
            MenuTitle(#sMenuTitleFile)
            MenuItem(#eMenuItemFileExit, #sMenuItemFileExit)
            MenuTitle(#sMenuTitleEdit)
            MenuTitle(#sMenuTitleHelp)
            MenuItem(#eMenuItemHelpLog, #sMenuItemHelpLog)
            MenuItem(#eMenuItemHelpAbout, #sMenuItemHelpAbout)

            BindMenuEvent(miMenuNumber, #eMenuItemFileExit, @EventMenuFileExit())
            BindMenuEvent(miMenuNumber, #eMenuItemHelpLog, @EventMenuHelpLog())
            BindMenuEvent(miMenuNumber, #eMenuItemHelpAbout, @EventMenuHelpAbout())
         EndIf

      Else
         DebugView::Error("Das Fenster >" + #PB_Compiler_Module + "< konnte nicht erstellt werden.")
      EndIf
      
      ; Meldungen in das Log-Fenster ausgeben.
      WindowLog::Write("Ausgabe mit 'WindowLog::Write()'.")
      WindowLog::Error("Ausgabe mit 'WindowLog::Error()'.")
      WindowLog::Stripline()

      DebugView::ProcOut()
      ProcedureReturn miWindowNumber
   EndProcedure


   Procedure Close()
      ; Schließt das Fenster.
      DebugView::ProcIn()

      If miWindowNumber
         UnbindEvent(#PB_Event_CloseWindow, @EventCloseWindow(), miWindowNumber)
         UnbindEvent(#PB_Event_MinimizeWindow, @EventMinimizeWindow(), miWindowNumber)
         UnbindEvent(#PB_Event_RestoreWindow, @EventRestoreWindow(), miWindowNumber)

         UnbindMenuEvent(miMenuNumber, #eMenuItemFileExit, @EventMenuFileExit())
         UnbindMenuEvent(miMenuNumber, #eMenuItemHelpLog, @EventMenuHelpLog())
         UnbindMenuEvent(miMenuNumber, #eMenuItemHelpAbout, @EventMenuHelpAbout())

         CloseWindow(miWindowNumber)
         miWindowNumber = #Null

         ; Ereignis für das Beenden des Programms an die Hauptschleife senden.
         PostEvent(#eCustomEventExitApplication)

      EndIf

      DebugView::ProcOut()
      ProcedureReturn
   EndProcedure


EndModule   ; WindowStart.
