﻿EnableExplicit

;- -- Information.
; Dieses Modul sollte in jedes Projekt mit >XIncludeFile "..\..\_ PureBasic\WindowLog-v01.pbi"<
; in die Hauptdatei, z.B. "Application.pb" eingebunden werden.

;- -- Version 01.
; Status:   Fertig.
; Autor:    Peter Dreisiebner
; Website:  http://peter.dreisiebner.at
; E-Mail:   web@dreisiebner.at
; Geändert: 2017-10-13 - PureBasic v5.61 (x86).

;- -- Verwendung.
; TODO - Anleitung für die Verwendung schreiben.
; TODO - Ausgabe der Meldungen sammeln und nach einer Sekunde ausgeben bzw. wenn der Buffer voll ist.

;-
;- -- DeclareModule WindowLog.
;
XIncludeFile "WindowLogDeclare-v01.pbi"


;-
;- -- Module WindowLog.
;
Module WindowLog
   EnableExplicit
   UseModule Common
   DebugOutWindowLog
   
   ;-
   ;- -- Konstanten.
   #iWindowLogWidth = 640
   #iWindowLogHeight = 360
   #sWindowLogTitleActive = " - Log"
   #sWindowLogTitleDeactive = #sWindowLogTitleActive + " (Deaktiviert)"

   #sFontName = "Courier New"
   #iFontSize = 9
   #sEditorGadgetPropertyOrgProc = "orgproc"

   #iMaxLines = 10000
   #sLineCounter = "0000"
   #iLineCounterWidth = 4
   #sInfoClearLog = "Die Ausgabe wird nach " + #iMaxLines + " Zeilen gelöscht."
   #sInfoError = "FEHLER: "
   #sInfoStripline = "------------------------------------------------------------"
   #sTimeFormat = "%hh:%ii:%ss"
   #sFileTypes = "Text (*.txt)|*.txt|Alle Dateien (*.*)|*.*"
   #iFileType = 0
   #sFileType = ".txt"

   #sMenuTitleFile = "&Datei"
   #sMenuItemFileSaveAs = ~"&Speichern unter...\tStrg+S"
   #sMenuItemFileClose = ~"Schli&eßen\tStrg+F4"
   #sMenuTitleEdit = "&Bearbeiten"
   #sMenuItemEditCopy = ~"&Kopieren\tStrg+C"
   #sMenuItemEditClear = ~"&Löschen\tStrg+L"
   #sMenuTitleLog = "&Log"
   #sMenuItemLogCaptureEvents = ~"&Ereignisse anzeigen\tStrg+E"
   Enumeration eMenuItemId
      #eMenuItemFileSaveAs
      #eMenuItemFileClose
      #eMenuItemEditCopy
      #eMenuItemEditClear
      #eMenuItemLogCaptureEvents
   EndEnumeration
   
   ;-
   ;- -- Variablen.
   Global miWindowInstanceCounter.i ; Zähler für die Instanz des Fensters.
   Global miWindowInvisibleNumber.i ; Enthält die Nummer des unsichtbaren Parent-Fensters.
   Global miWindowNumber.i          ; Enthält die Nummer des Fensters.
   Global mfWindowInvisible.i       ; Ist True wenn das unsichtbaren Parent-Fenster erstellt wurde.
   Global mfWindow.i                ; Ist True wenn das Fenster erstellt wurde.
   Global mfWindowCentered.i        ; Ist True wenn das Fenster zentriert angezeigt werden soll.
   Global mfRestoreWindow.i         ; Ist True wenn das Fenster angezeigt werden soll, wenn das Programm wiederhergestellt wird.
   Global miMenuNumber.i            ; Enthält die Nummer des Menüs.
   Global miMenuPopupNumber.i       ; Enthält die Nummer des Popup-Menüs im Editor-Gadget.
   Global mfCaptureEvents.i         ; Ist True wenn die Ereignisse angezeigt werden.
   Global miEditorGadgetNumber.i    ; Enthält die Nummer des Editor-Gadgets.
   Global miLineNumber.i            ; Enthält die nächste Zeilennummer des Log.
   Global miCharCounter.i           ; Zähler für das Positionieren des Carets im Editor-Gadget.
   
   ;-

   Procedure EventCloseWindow()
      ; Ereignisprozedur wenn das Fenster geschlossen werden soll.
      ; Das Fenster wird nur ausgeblendet.
      DebugView::ProcIn()

      If miWindowNumber
         Hide()
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure EventSizeWindow()
      ; Ereignisprozedur wenn das Fenster in der Größe geändert wurde.
      ; Das Editor-Gadget an die neue Größe des Fensters anpassen.
      DebugView::ProcIn()
      
      If miWindowNumber
         If miEditorGadgetNumber
            ResizeGadget(miEditorGadgetNumber, #PB_Ignore, #PB_Ignore,
                         WindowWidth(miWindowNumber), WindowHeight(miWindowNumber) - MenuHeight())
         EndIf
      EndIf
      
      DebugView::ProcOut()
   EndProcedure


   Procedure EventMenuFileSaveAs()
      DebugView::ProcIn()
      Protected sFile.s
      Protected iFileNumber.i
      Protected fResult.i

      sFile = SaveFileRequester("Speichern unter", #sApplicationName + #sFileType, #sFileTypes, #iFileType)

      fResult = Bool(sFile)
      If fResult
         iFileNumber = CreateFile(#PB_Any, sFile, #PB_UTF8)
         fResult = Bool(iFileNumber)
         If (Not fResult)
            Error("Die Datei >" + sFile + "< konnte nicht erstellt werden.")
         EndIf
      EndIf

      If fResult
         fResult = WriteStringFormat(iFileNumber, #PB_UTF8)
         If (Not fResult)
            Error("Das 'Byte Order Mark' konnte nicht in die Datei >" + sFile + "< geschrieben werden.")
         EndIf
      EndIf

      If fResult
         fResult = WriteString(iFileNumber, GetGadgetText(miEditorGadgetNumber), #PB_UTF8)
         If (Not fResult)
            Error("Die Logdaten konnten nicht in die Datei >" + sFile + "< geschrieben werden.")
         EndIf
      EndIf

      If fResult
         Write("Die Logdaten wurden in der Datei >" + sFile + "< gespeichert.")
      EndIf

      If iFileNumber
         CloseFile(iFileNumber)
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure EventMenuFileClose()
      DebugView::ProcIn()

      Hide()

      DebugView::ProcOut()
   EndProcedure


   Procedure EventMenuEditCopy()
      DebugView::ProcIn()

      Copy()

      DebugView::ProcOut()
   EndProcedure


   Procedure EventMenuEditClear()
      DebugView::ProcIn()

      Clear()

      DebugView::ProcOut()
   EndProcedure


   Procedure EventMenuLogCaptureEvents()
      DebugView::ProcIn()

      mfCaptureEvents = Bool(Not mfCaptureEvents)
      SetMenuItemState(miMenuNumber, #eMenuItemLogCaptureEvents, mfCaptureEvents)

      If mfCaptureEvents
         SetWindowTitle(miWindowNumber, #sApplicationName + #sWindowLogTitleActive)
      Else
         SetWindowTitle(miWindowNumber, #sApplicationName + #sWindowLogTitleDeactive)
      EndIf

      DebugView::ProcOut()
   EndProcedure


   Procedure EditorGadgetWindowProcedure(ihWnd.i, iMsg.i, iwParam.i, ilParam.i)
      ; Fensterprozedur für das Editor-Gadget.
      ; Wird für das Anzeigen des Popup-Menüs verwendet.
      ;;DebugView::ProcIn() : CompilerError "Nicht aktivieren, rekursiv, Stack overflow!"
      Protected iOriginalProcedure.i

      iOriginalProcedure = GetProp_(ihWnd, #sEditorGadgetPropertyOrgProc)

      Select iMsg
         Case #WM_NCDESTROY
            RemoveProp_(ihWnd, #sEditorGadgetPropertyOrgProc)

         Case #WM_RBUTTONUP
            DisplayPopupMenu(miMenuPopupNumber, WindowID(miWindowNumber))
            
         Case #WM_CHAR
            If (iwParam = #VK_ESCAPE)
               Close(#False)
            EndIf

      EndSelect

      ;;DebugView::ProcOut() : CompilerError "Nicht aktivieren, rekursiv, Stack overflow!"
      ProcedureReturn CallWindowProc_(iOriginalProcedure, ihWnd, iMsg, iwParam, ilParam)
   EndProcedure


   Procedure Load(iParentWindowId.i = #Null)
      ; Das Fenster erstellen aber nicht anzeigen.
      ; -> iParentWindowId:   Die ID des Parent-Fenster. Das Log-Fenster bleibt dann immer im Vordergrund.
      DebugView::ProcIn(miWindowInstanceCounter)
      Protected iFontNumber.i, iGadgetId.i

      ; Das Log-Fenster nur einmal beim ersten Aufruf erstellen.
      If (mfWindow = #False)
         If (iParentWindowId = #Null)
            miWindowInvisibleNumber = OpenWindow(#PB_Any, #PB_Ignore, #PB_Ignore, #iWindowLogWidth, #iWindowLogHeight,
                                                 #sApplicationName + #sWindowLogTitleActive + " (Parent)", #PB_Window_Invisible)
            If miWindowInvisibleNumber
               mfWindowInvisible = #True
               iParentWindowId = WindowID(miWindowInvisibleNumber)
            Else
               mfWindowInvisible = #False
               iParentWindowId = #Null
            EndIf
         EndIf

         miWindowNumber = OpenWindow(#PB_Any, #PB_Ignore, #PB_Ignore, #iWindowLogWidth, #iWindowLogHeight,
                                     #sApplicationName + #sWindowLogTitleActive,
                                     #PB_Window_MaximizeGadget | #PB_Window_SizeGadget |
                                     #PB_Window_Invisible, iParentWindowId)
         If miWindowNumber
            mfWindow = #True
            mfWindowCentered = #True
            miWindowInstanceCounter + 1
            mfCaptureEvents = #True
            miLineNumber = 0
            miCharCounter = 0

            BindEvent(#PB_Event_CloseWindow, @EventCloseWindow(), miWindowNumber)
            BindEvent(#PB_Event_SizeWindow, @EventSizeWindow(), miWindowNumber)

            Tool::WindowReduceFlicker(miWindowNumber)

            miMenuNumber = CreateMenu(#PB_Any, WindowID(miWindowNumber))
            If miMenuNumber
               MenuTitle(#sMenuTitleFile)
               MenuItem(#eMenuItemFileSaveAs, #sMenuItemFileSaveAs)
               MenuItem(#eMenuItemFileClose, #sMenuItemFileClose)
               MenuTitle(#sMenuTitleEdit)
               MenuItem(#eMenuItemEditCopy, #sMenuItemEditCopy)
               MenuItem(#eMenuItemEditClear, #sMenuItemEditClear)
               MenuTitle(#sMenuTitleLog)
               MenuItem(#eMenuItemLogCaptureEvents, #sMenuItemLogCaptureEvents)
               SetMenuItemState(miMenuNumber, #eMenuItemLogCaptureEvents, mfCaptureEvents)

               BindMenuEvent(miMenuNumber, #eMenuItemFileSaveAs, @EventMenuFileSaveAs())
               BindMenuEvent(miMenuNumber, #eMenuItemFileClose, @EventMenuFileClose())
               BindMenuEvent(miMenuNumber, #eMenuItemEditCopy, @EventMenuEditCopy())
               BindMenuEvent(miMenuNumber, #eMenuItemEditClear, @EventMenuEditClear())
               BindMenuEvent(miMenuNumber, #eMenuItemLogCaptureEvents, @EventMenuLogCaptureEvents())

               AddKeyboardShortcut(miWindowNumber, #PB_Shortcut_Control | #PB_Shortcut_S, #eMenuItemFileSaveAs)
               AddKeyboardShortcut(miWindowNumber, #PB_Shortcut_Control | #PB_Shortcut_F4, #eMenuItemFileClose)
               AddKeyboardShortcut(miWindowNumber, #PB_Shortcut_Control | #PB_Shortcut_C, #eMenuItemEditCopy)
               AddKeyboardShortcut(miWindowNumber, #PB_Shortcut_Control | #PB_Shortcut_L, #eMenuItemEditClear)
               AddKeyboardShortcut(miWindowNumber, #PB_Shortcut_Control | #PB_Shortcut_E, #eMenuItemLogCaptureEvents)
            EndIf

            ; Die Menüeinträge vom normalen Menü für das Popup-Menü verwenden.
            miMenuPopupNumber = CreatePopupMenu(#PB_Any)
            If miMenuPopupNumber
               MenuItem(#eMenuItemEditCopy, #sMenuItemEditCopy)
               MenuItem(#eMenuItemEditClear, #sMenuItemEditClear)
            EndIf

            miEditorGadgetNumber = EditorGadget(#PB_Any, 0, 0, WindowWidth(miWindowNumber), WindowHeight(miWindowNumber))
            If miEditorGadgetNumber
               iFontNumber = LoadFont(#PB_Any, #sFontName, #iFontSize)
               SetGadgetFont(miEditorGadgetNumber, FontID(iFontNumber))
               Tool::EditorGadgetAllowOnlyPlaintext(miEditorGadgetNumber)
               ; Die WindowProcedure für das Popup-Menü mit eigener Prozedur ersetzen.
               iGadgetId = GadgetID(miEditorGadgetNumber)
               SetProp_(iGadgetId, #sEditorGadgetPropertyOrgProc,
                        SetWindowLongPtr_(iGadgetId, #GWLP_WNDPROC, @EditorGadgetWindowProcedure()))
               ; Hinweis ausgeben, dass die Ausgaben nach z.B. 10000 Zeilen gelöscht werden.
               Clear(#True)
            EndIf

         EndIf
      EndIf

      DebugView::ProcOut(miWindowInstanceCounter)
      ProcedureReturn
   EndProcedure


   Procedure Close(fClose.i)
      ; Blendet das Fenster aus, oder schließt es wenn True übergeben wird.
      ; -> fClose:   True wenn das Fenster geschloßen werden soll. Bei False wird es nur versteckt.
      DebugView::ProcIn(miWindowInstanceCounter)

      If mfWindow
         If fClose
            mfWindow = #False

            UnbindEvent(#PB_Event_CloseWindow, miWindowNumber)
            UnbindEvent(#PB_Event_SizeWindow, miWindowNumber)

            RemoveKeyboardShortcut(miWindowNumber, #PB_Shortcut_All)

            If miMenuNumber
               UnbindMenuEvent(miMenuNumber, #eMenuItemFileSaveAs, @EventMenuFileSaveAs())
               UnbindMenuEvent(miMenuNumber, #eMenuItemFileClose, @EventMenuFileClose())
               UnbindMenuEvent(miMenuNumber, #eMenuItemEditCopy, @EventMenuEditCopy())
               UnbindMenuEvent(miMenuNumber, #eMenuItemEditClear, @EventMenuEditClear())
               UnbindMenuEvent(miMenuNumber, #eMenuItemLogCaptureEvents, @EventMenuLogCaptureEvents())
               FreeMenu(miMenuNumber)
               miMenuNumber = #Null
            EndIf

            If miMenuPopupNumber
               FreeMenu(miMenuPopupNumber)
               miMenuPopupNumber = #Null
            EndIf

            CloseWindow(miWindowNumber)
            miWindowNumber = #Null

         Else
            Hide()
         EndIf
      EndIf

      If mfWindowInvisible
         If fClose
            mfWindowInvisible = #False
            If IsWindow(miWindowInvisibleNumber)
               CloseWindow(miWindowInvisibleNumber)
               miWindowInvisibleNumber = #Null
            EndIf
         EndIf
      EndIf

      DebugView::ProcOut(miWindowInstanceCounter)
   EndProcedure


   Procedure Show(iParentWindowNumber.i = #Null)
      ; Zeigt das Fenster an.
      ; -> iParentWindowNumber:  Die Nummer des Parent-Fenster. Das Log-Fenster wird dann einmalig zentriert angezeigt.
      ; Rückgabe:                Die Nummer des Log-Fensters oder Null bei einem Fehler.
      DebugView::ProcIn(miWindowInstanceCounter)

      If mfWindow
         If mfWindowCentered
            mfWindowCentered = #False
            If (mfWindowInvisible And iParentWindowNumber)
               ResizeWindow(miWindowInvisibleNumber, WindowX(iParentWindowNumber), WindowY(iParentWindowNumber),
                            WindowWidth(iParentWindowNumber), WindowHeight(iParentWindowNumber))
            EndIf
            HideWindow(miWindowNumber, #False, #PB_Window_WindowCentered)

         Else
            HideWindow(miWindowNumber, #False)
         EndIf

         SetActiveGadget(miEditorGadgetNumber)
      EndIf

      DebugView::ProcOut(miWindowInstanceCounter)
   EndProcedure


   Procedure Hide()
      ; Blendet das Fenster aus.
      DebugView::ProcIn(miWindowInstanceCounter)

      If mfWindow
         mfRestoreWindow = IsWindowVisible_(WindowID(miWindowNumber))
         HideWindow(miWindowNumber, #True)
      EndIf

      DebugView::ProcOut(miWindowInstanceCounter)
   EndProcedure


   Procedure RestoreWindow()
      ; Zeigt das Fenster an, wenn das Programm wiederhergestellt wird.
      DebugView::ProcIn(miWindowInstanceCounter)

      If mfWindow
         If mfRestoreWindow
            HideWindow(miWindowNumber, #False)
         EndIf
      EndIf

      DebugView::ProcOut(miWindowInstanceCounter)
   EndProcedure


   Procedure Write(sText.s)
      ; Gibt den übergebenen Text im Editor-Gadget aus.
      ;;DebugView::ProcIn() : CompilerError "Nicht aktivieren, rekursiv, Stackoverflow!"

      If (mfWindow And mfCaptureEvents)
         If (miLineNumber = #iMaxLines)
            Clear(#True)
         EndIf

         If miEditorGadgetNumber
            sText = Right(#sLineCounter + Str(miLineNumber), #iLineCounterWidth) +
                    " [" + FormatDate(#sTimeFormat, Date()) + "] " + sText
            AddGadgetItem(miEditorGadgetNumber, -1, sText)

            SendMessage_(GadgetID(miEditorGadgetNumber), #EM_SETSEL, miCharCounter, miCharCounter)
            miCharCounter + Len(sText) + 1
         EndIf

         miLineNumber + 1
      EndIf

      ;;DebugView::ProcOut() : CompilerError "Nicht aktivieren, rekursiv, Stackoverflow!"
   EndProcedure


   Procedure Error(sText.s)
      ; Gibt den übergebenen Text im Editor-Gadget aus.
      DebugView::ProcIn()

      Write(#sInfoError + sText)

      DebugView::ProcOut()
   EndProcedure


   Procedure Stripline()
      ; Gibt eine Trennlinie im Editor-Gadget aus.
      DebugView::ProcIn()

      Write(#sInfoStripline)

      DebugView::ProcOut()
   EndProcedure


   Procedure Clear(fInfo.i = #False)
      ; Löscht den Text im Editor-Gadget.
      ;;DebugView::ProcIn() : CompilerError "Nicht aktivieren, rekursiv!"

      If miEditorGadgetNumber
         miLineNumber = 0
         miCharCounter = 0
         ClearGadgetItems(miEditorGadgetNumber)

         If fInfo
            Write(#sInfoClearLog)
            miLineNumber = 0
            Stripline()
         EndIf
      EndIf

      ;;DebugView::ProcOut() : CompilerError "Nicht aktivieren, rekursiv!"
   EndProcedure


   Procedure Copy()
      ; Kopiert den ausgewählten oder gesamten Text vom Editor-Gadget in die Zwischenablage.
      DebugView::ProcIn()
      Protected iStartPos.i, iStopPos.i

      If miEditorGadgetNumber
         ClearClipboard()
         SendMessage_(GadgetID(miEditorGadgetNumber), #EM_GETSEL, @iStartPos, @iStopPos)
         If ((iStopPos - iStartPos) > 0)
            SendMessage_(GadgetID(miEditorGadgetNumber), #WM_COPY, #Null, #Null)
         Else
            SetClipboardText(GetGadgetText(miEditorGadgetNumber))
         EndIf
      EndIf

      DebugView::ProcOut()
   EndProcedure


EndModule   ; WindowLog.
